function ImgOut = embedmask(varargin)
%EMBEDMASK Embed a colored mask into an image
% 
% IMGOUT = EMBEDMASK(IMGIN,MASK,COLOR)
% Superposes the binary image MASK on the image IMGIN -
% each pixel of value 1 in MASK will turn the pixel with
% the same coordinates in IMGIN to the color specified in
% COLOR.
% 
% It is possible to obtain similar results by displaying
% IMGIN, then using 'plot' to superpose MASK over it.
% But this is on visualy identic to EMBEDMASK, since you 
% will still be left with two distinct images. If however
% you need just one file providing the superposition (for
% printing for example), then you may use EMBEDMASK.
% 
% Class support
% -------------
% IMGIN must be uint8 and it should not be an indexed 
% image (convert with IND2RGB from Image Processing Toolbox); 
% MASK may be of any class, but must contain only 0's and 1's.
% 
% BIG BUG
% -------
% I can't get class support for other than uint8.
% In some images the contrast is lowered.
% Function OK on a gray or color JPG/TIF.

% Vlad Atanasiu - 07/09/2003
% Revisions: 0  > 00/00/0000


% Parse inputs
try
    [ImgIn,Mask,color] = ParseInputs(varargin{:});
catch
    rethrow(lasterror);
end

% get maximum value of color range
imageClass = class(ImgIn);
switch imageClass
    case 'uint8'
        maxColor = 2^8-1;
%     case 'uint16'
%         maxColor = 2^16-1;
%     case {'uint32','uint64'}
%         maxColor = 2^24-1;
%     case {'double','single'}
%         if length(size(ImgIn)) == 2
%             maxColor = max(max(ImgIn))-1;
%         else
%             maxColor = max(max(max(ImgIn)))-1;
%         end
    otherwise
        % catch unsuported data class
        error('AVe:Images:embedmask:classNotSuported',...
            'Suported data classes are: uint8.')
end

% change to double precision so that math operations are possible
ImgIn = double(ImgIn);
Mask = double(Mask);
MaskNeg = double(~Mask);

% make filtered image from negative mask
ImgIn(:,:,1) = ImgIn(:,:,1).*MaskNeg;
if length(size(ImgIn)) == 3
    ImgIn(:,:,2) = ImgIn(:,:,2).*MaskNeg;
    ImgIn(:,:,3) = ImgIn(:,:,3).*MaskNeg;
else
    ImgIn(:,:,2) = ImgIn(:,:,1);
    ImgIn(:,:,3) = ImgIn(:,:,1);
end

% color mask
Mcolor = [];
Mcolor(:,:,1) = Mask.*color(1)*maxColor;
Mcolor(:,:,2) = Mask.*color(2)*maxColor;
Mcolor(:,:,3) = Mask.*color(3)*maxColor;

% add filtered image to colored mask
ImgOut = [];
ImgOut(:,:,1) = ImgIn(:,:,1) + Mcolor(:,:,1);
ImgOut(:,:,2) = ImgIn(:,:,2) + Mcolor(:,:,2);
ImgOut(:,:,3) = ImgIn(:,:,3) + Mcolor(:,:,3);

% restore color class to that of the input image
switch imageClass
    case 'uint8'
        ImgOut = uint8(ImgOut);
    case 'uint16'
        ImgOut = uint16(ImgOut);
    case 'uint32'
        ImgOut = uint32(ImgOut);
    case 'uint64'
        ImgOut = uint64(ImgOut);
    case 'single'
        ImgOut = single(ImgOut);
end


% =======================================================
% Subfunction ParseInputs
% -------------------------------------------------------
function [ImgIn,Mask,color] = ParseInputs(varargin)

% initialisation
ImgIn = [];
Mask = [];
color = [];

switch nargin
case 0
    % EMBEDMASK
    
    error('AVe:Images:embedmask:noInput',...
        'Please supply input data.');
case 1
    % EMBEDMASK(IMGIN)
    
    error('AVe:Images:embedmask:noMaskInput',...
        'Please supply a ''mask'' image.');
case 2
    % EMBEDMASK(IMGIN,MASK)
    
    if min(min(Mask)) ~= 0 | max(max(Mask)) ~= 1
    % check MASK for binarity
        error('AVe:Images:embedmask:notBinaryMask',...
            'MASK must contain only 0''s & 1''s');
    elseif size(ImgIn,1) ~= size(Mask,1) | size(ImgIn,2) ~= size(Mask,2)
    % IMGIN & MASK must have same height & width
        error('AVe:Images:embedmask:imagesNotSameSize',...
            'Image & mask must have same height & width.');
    else
        % Input OK
        ImgIn = varargin{1};
        Mask = varargin{2};
        
        % set default color
        color = [1 0 0];    % red
    end    
case 3
    % EMBEDMASK(IMGIN,MASK,COLOR)
    
    flag = 0;
    
    if (ischar(varargin{3}))
        % define colors
        colorNames.long = {...
                'yellow',...
                'magenta',...
                'cyan',...
                'red',...
                'green',...
                'blue',...
                'white',...
                'black',...
            };
        colorNames.short = {'y','m','c','r','g','b','w','k',};
        colorNames.rgb = {...
                [1 1 0],...
                [1 0 1],...
                [0 1 1],...
                [1 0 0],...
                [0 1 0],...
                [0 0 1],...
                [1 1 1],...
                [0 0 0],...
            };
        
        % check strings
        flag = 1;
        for k = 1:size(colorNames.long,2)
            if varargin{3} == colorNames.short{k} | ...
                    strcmp(varargin{3},colorNames.long{k}) == 1
                % convert to RGB
                color = colorNames.rgb{k};
                flag = 0;
                break
            end
        end
    else
        if isempty(color) | (size(color,1) ~=1 & size(color,2) ~=3)
            % no COLOR input
            flag = 1;
        else
            % check for numeric validity
            % --- CODE NOT WRITEN ---
        end
    end
    
    if flag == 0
        % Input OK
        ImgIn = varargin{1};
        Mask = varargin{2};
    else
        error('AVe:Images:embedmask:wrongColorInput',...
            'Color-input must be a 1x3 array with values 0 <= => 1,\n or standard Matlab color names, like ''red'' or ''r''.')
    end
end


